;; dans cette version les expression en forme normale conjectives sont interprétées
;; les valeurs des variables sont dans une liste d'association

;; calcule la valeur de vérité de clause
(define (clause v L)
  (if (null? L)
      #f
      (if (equal? 'n (car L)) ; une négation ?
          (let ( (x (assoc (car (cdr L)) v)) )  ; variable de la clause
            (if (cadr x)
                (clause v (cdr (cdr L)))
                #t))
          (let ((x (assoc (car L) v)))
              (if (cadr x)
                  #t
                  (clause v (cdr L)))))))

(clause '( (1 #t) (2 #f)) '(n 1 2))
(clause '( (1 #t) (2 #t)) '(n 1 2))

;; calcule la valeur de vérité de la formule logique écrite en CNF :
;; ( (clause 1) (clause 2) ... )
(define (formule v LL)
  (if (null? LL)
      #t
      (and (clause v (car LL))
           (formule v (cdr LL)))))

(formule '((1 #t) (2 #f)) '((n 1 2) (1 2)))

(formule '((1 #t) (2 #t)) '((n 1 2) (1 2)))


;; explore en profondeur d'abord toutes les solutions possible
;; f est la formule logique
;; n le nombre de variable à instancier
;; L la liste des variables déjà instanciées
(define (explorer n f)
  (define (expl n f v)
    (if (= n 0)
        (if (formule v f)
            (cons v '())   ;; une seule solution dans la liste résultat
            '())           ;; aucune solution dans la liste résultat
        (append  
         (expl (- n 1) f (cons (list n #t) v))     ;; réunion des solutions regroupant celle où la variable n est vraie  
         (expl (- n 1) f (cons (list n #f) v)))))  ;; et celle où elle est fausse
  (expl n f '()))

(explorer 2 '((n 1 2) (1 2)))

; 3 eme exemple du cours
(print "exemple à 5 variables du cours")
(explorer 5 '((4 2 3) (2 4 5) (n 2 1 5) (n 2 5 4) (1 n 3 n 4)))
